@@ -22,23 +22,26 @@ import errno
 import hashlib
 import logging
 import os
+import stat
 import urlparse
 
 from oslo.config import cfg
+from oslo.serialization import jsonutils
+from oslo.utils import excutils
+from oslo.utils import units
 
 import glance_store
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _
+from glance_store import i18n
 import glance_store.location
-from glance_store.openstack.common import excutils
-from glance_store.openstack.common import jsonutils
-from glance_store.openstack.common import processutils
-from glance_store.openstack.common import units
 
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LE = i18n._LE
+_LW = i18n._LW
 
 _FILESYSTEM_CONFIGS = [
     cfg.StrOpt('filesystem_store_datadir',
@@ -51,7 +54,16 @@ _FILESYSTEM_CONFIGS = [
                help=_("The path to a file which contains the "
                       "metadata to be returned with any location "
                       "associated with this store.  The file must "
-                      "contain a valid JSON dict."))]
+                      "contain a valid JSON dict.")),
+    cfg.IntOpt('filesystem_store_file_perm',
+               default=0,
+               help=_("The required permission for created image file. "
+                      "In this way the user other service used, e.g. Nova, "
+                      "who consumes the image could be the exclusive member "
+                      "of the group that owns the files created. Assigning "
+                      "it less then or equal to zero means don't change the "
+                      "default permission of the file. This value will be "
+                      "decoded as an octal digit."))]
 
 
 class StoreLocation(glance_store.location.StoreLocation):
@@ -88,19 +100,25 @@ class ChunkedFile(object):
     something that can iterate over a large file
     """
 
-    def __init__(self, filepath, chunk_size=None):
+    def __init__(self, filepath, offset=0, chunk_size=None, partial=False):
+        self.partial = partial
         self.filepath = filepath
         self.chunk_size = chunk_size
         self.fp = open(self.filepath, 'rb')
+        if offset:
+            self.fp.seek(offset)
 
     def __iter__(self):
-        """Return an iterator over the image file"""
+        """Return an iterator over the image file."""
         try:
             if self.fp:
                 while True:
                     chunk = self.fp.read(self.chunk_size)
                     if chunk:
                         yield chunk
+
+                        if self.partial:
+                            break
                     else:
                         break
         finally:
@@ -136,6 +154,35 @@ class Store(glance_store.driver.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name="filesystem", reason=msg)
 
+    def _set_exec_permission(self, datadir):
+        """
+        Set the execution permission of owner-group and/or other-users to
+        image directory if the image file which contained needs relevant
+        access permissions.
+
+        :datadir is a directory path in which glance writes image files.
+        """
+
+        if self.conf.glance_store.filesystem_store_file_perm <= 0:
+            return
+
+        try:
+            mode = os.stat(datadir)[stat.ST_MODE]
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            if perm & stat.S_IRWXO > 0:
+                if not mode & stat.S_IXOTH:
+                    # chmod o+x
+                    mode |= stat.S_IXOTH
+                    os.chmod(datadir, mode)
+            if perm & stat.S_IRWXG > 0:
+                if not mode & stat.S_IXGRP:
+                    # chmod g+x
+                    os.chmod(datadir, mode | stat.S_IXGRP)
+        except (IOError, OSError):
+            LOG.warn(_LW("Unable to set execution permission of owner-group "
+                         "and/or other-users to datadir: %s") % datadir)
+
     def _create_image_directories(self, directory_paths):
         """
         Create directories to write image files if
@@ -147,6 +194,7 @@ class Store(glance_store.driver.Store):
         for datadir in directory_paths:
             if os.path.exists(datadir):
                 self._check_write_permission(datadir)
+                self._set_exec_permission(datadir)
             else:
                 msg = _("Directory to write image files does not exist "
                         "(%s). Creating.") % datadir
@@ -154,6 +202,7 @@ class Store(glance_store.driver.Store):
                 try:
                     os.makedirs(datadir)
                     self._check_write_permission(datadir)
+                    self._set_exec_permission(datadir)
                 except (IOError, OSError):
                     if os.path.exists(datadir):
                         # NOTE(markwash): If the path now exists, some other
@@ -161,6 +210,7 @@ class Store(glance_store.driver.Store):
                         # But it doesn't hurt, so we can safely ignore
                         # the error.
                         self._check_write_permission(datadir)
+                        self._set_exec_permission(datadir)
                         continue
                     reason = _("Unable to create datadir: %s") % datadir
                     LOG.error(reason)
@@ -191,6 +241,19 @@ class Store(glance_store.driver.Store):
             raise exceptions.BadStoreConfiguration(store_name="filesystem",
                                                    reason=reason)
 
+        if self.conf.glance_store.filesystem_store_file_perm > 0:
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            if not perm & stat.S_IRUSR:
+                reason = _LE("Specified an invalid "
+                             "'filesystem_store_file_perm' option which "
+                             "could make image file to be unaccessible by "
+                             "glance service.")
+                LOG.error(reason)
+                reason = _("Invalid 'filesystem_store_file_perm' option.")
+                raise exceptions.BadStoreConfiguration(store_name="filesystem",
+                                                       reason=reason)
+
         self.multiple_datadirs = False
         directory_paths = set()
         if self.conf.glance_store.filesystem_store_datadir:
@@ -315,8 +378,10 @@ class Store(glance_store.driver.Store):
         msg = _("Found image at %s. Returning in ChunkedFile.") % filepath
         LOG.debug(msg)
         return (ChunkedFile(filepath,
-                            chunk_size=chunk_size or self.READ_CHUNKSIZE),
-                filesize)
+                            offset=offset,
+                            chunk_size=chunk_size or self.READ_CHUNKSIZE,
+                            partial=chunk_size is not None),
+                chunk_size or filesize)
 
     def get_size(self, location, context=None):
         """
@@ -362,10 +427,8 @@ class Store(glance_store.driver.Store):
         """
 
         #Calculate total available space
-        df = processutils.execute("df", "-k",
-                                  mount_point)[0].strip("'\n'")
-        total_available_space = int(df.split('\n')[1].split()[3]) * units.Ki
-
+        stvfs_result = os.statvfs(mount_point)
+        total_available_space = stvfs_result.f_bavail * stvfs_result.f_bsize
         return max(0, total_available_space)
 
     def _find_best_datadir(self, image_size):
@@ -460,6 +523,16 @@ class Store(glance_store.driver.Store):
                   {'bytes_written': bytes_written,
                    'filepath': filepath,
                    'checksum_hex': checksum_hex})
+
+        if self.conf.glance_store.filesystem_store_file_perm > 0:
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            try:
+                os.chmod(filepath, perm)
+            except (IOError, OSError):
+                LOG.warn(_LW("Unable to set permission to image: %s") %
+                         filepath)
+
         return ('file://%s' % filepath, bytes_written, checksum_hex, metadata)
 
     @staticmethod
@@ -469,4 +542,4 @@ class Store(glance_store.driver.Store):
         except Exception as e:
             msg = _('Unable to remove partial image '
                     'data for image %(iid)s: %(e)s')
-            LOG.error(msg % dict(iid=iid, e=e))
+            LOG.error(msg % dict(iid=iid, e=utils.exception_to_str(e)))
